//+--------------------------------------------------------------------------+
//|                                                                          |
//|                               Fahrenheit                                 |
//|                          BassBox OpenGL plugin                           |
//|                                                                          |
//|                              Version 2.00                                |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|                                                                          |
//|                         copyright(c) 2007-2017                           |
//|                                                                          |
//|                Patrice Terrier http://www.zapsolution.com                |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                  Project started on : 10-30-2007 (MM-DD-YYYY)            |
//|                        Last revised : 02-18-2017 (MM-DD-YYYY)            |
//+--------------------------------------------------------------------------+

//#include <windows.h>
//#include <mfapi.h>
//#include <gl/GL.h>
//#include <gl/GLU.h>
//#include <shlwapi.h>
//#include <math.h>
//#include "gl2.h"

#include <windows.h>
//#include "..\TCLib\stdio.h"
//#include "..\TCLib\string.h"
//#include "..\TCLib\stdlib.h"
//#include "..\TCLib\memory.h"
//#include "..\TCLib\malloc.h"
#include "..\TCLib\math.h"
//#include "..\TCLib\tchar.h"
#include "..\TCLib\Strings.cpp"

#include <gl/GL.h>
#include <gl/GLU.h>
#include "gl2.h"

typedef ULONGLONG QWORD;
#include "BBplugin.h"
#define ExportC extern "C" _declspec (dllexport)

long g_color[32];
GLubyte gnR[32], gnG[32], gnB[32];

void bbp_ColorInit() {
    g_color[0]  = bbpARGB(255, 0,44,233);  
    g_color[1]  = bbpARGB(255, 0,67,210);  
    g_color[2]  = bbpARGB(255, 0,89,187);  
    g_color[3]  = bbpARGB(255, 0,112,164); 
    g_color[4]  = bbpARGB(255, 0,135,142); 
    g_color[5]  = bbpARGB(255, 0,159,117); 
    g_color[6]  = bbpARGB(255, 0,183,88);  
    g_color[7]  = bbpARGB(255, 0,207,58);  
    g_color[8]  = bbpARGB(255, 0,231,29);  
    g_color[9]  = bbpARGB(255, 26,234,26); 
    g_color[10] = bbpARGB(255, 52,237,23); 
    g_color[11] = bbpARGB(255, 79,240,20); 
    g_color[12] = bbpARGB(255, 105,243,17);
    g_color[13] = bbpARGB(255, 126,245,14);
    g_color[14] = bbpARGB(255, 147,248,11);
    g_color[15] = bbpARGB(255, 168,250,8); 
    g_color[16] = bbpARGB(255, 189,253,5); 
    g_color[17] = bbpARGB(255, 210,255,2); 
    g_color[18] = bbpARGB(255, 233,255,0); 
    g_color[19] = bbpARGB(255, 255,255,0); 
    g_color[20] = bbpARGB(255, 255,251,0); 
    g_color[21] = bbpARGB(255, 255,235,0); 
    g_color[22] = bbpARGB(255, 255,215,0); 
    g_color[23] = bbpARGB(255, 255,196,0); 
    g_color[24] = bbpARGB(255, 255,176,0); 
    g_color[25] = bbpARGB(255, 255,156,0); 
    g_color[26] = bbpARGB(255, 253,137,0); 
    g_color[27] = bbpARGB(255, 255,117,0); 
    g_color[28] = bbpARGB(255, 255,97,0);  
    g_color[29] = bbpARGB(255, 255,78,0);  
    g_color[30] = bbpARGB(255, 255,58,0);  
    g_color[31] = bbpARGB(255, 255,0,0);   
}

void MakeColorLUT() {
    COLORBYTES bytestruct;
    for (long K = 0; K < 32; K++) {
        MoveMemory(&bytestruct, &g_color[K], sizeof(COLORBYTES));
        gnR[K] = bytestruct.R;
        gnG[K] = bytestruct.G;
        gnB[K] = bytestruct.B;
    }
}

void DrawCylinder(IN float Radius, IN float Value, IN float R1, IN float R2) {
    GLUquadric* quadObj = gluNewQuadric(); // Pointer to the Quadric Object (Return 0 If No Memory))
    if (quadObj) {
        gluQuadricNormals(quadObj, GLU_SMOOTH); // Create Smooth Normals
        gluQuadricTexture(quadObj, GL_TRUE);    // Create Texture Coords
        gluCylinder(quadObj, Radius * R1, Radius * R2, Value, 24, 1);
        gluDeleteQuadric(quadObj);
    }
}

ExportC long BBProc (OUT BBPLUGIN &BBP) {
    long ShowLogo = 0, K = 0, nRet = BBP_SUCCESS;
    static BBPTEXTURE mt[2];
    GLbyte nValue;

    static float PulseAngle, WasPulse;
    static long TT;
    float pulse, rAspect;  
    float pv[128];
    long c = 0;

    const float UseSize = 6.0f;

    switch (BBP.msg) {
    case BBP_RENDER:
        // Velocity section
        nValue = (GLbyte)max(BBP.lpeak / 700, BBP.rpeak / 700);
        pulse = (float) nValue / 28.0f;

        PulseAngle = (float)fmod(PulseAngle + pulse, 360.0f);
        if ((PulseAngle > 90.0f) && (WasPulse < 90.0f)) {
            TT++; if (TT > 1) { TT = 0; }
        }
        if (TT == 0) { TT = 1; }

        WasPulse = PulseAngle;

        short wd1, wd2;
        for (K = 0; K < 128; K++) {
            // WIM MAGIC!
            // BBP.WIMdata returns a pointer to an area of memory
            // containing an array of 256 short integer.
            // Warning, the Left and Right audio channels are mixed together
            // thus we step 4, to consider only one channel.
            MoveMemory(&wd1, BBP.wimdata + K * 4, 2);
            MoveMemory(&wd2, BBP.wimdata + 2 + K * 4, 2);
            pv[K] = (float) max((float)wd1 / 32767.0f, (float)wd2 / 32767.0f);
            if ((ShowLogo == 0) && (pv[K])) { ShowLogo = -1; }
        }
        ShowLogo = !ShowLogo;

        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        glLoadIdentity();



        //glDisable(GL_LIGHTING);
        // Draw the background texture ------------------------
        glBindTexture(GL_TEXTURE_2D, mt[0].Texture);
        //glDisable(GL_BLEND);
        glBegin(GL_QUADS);
           glColor4ub(nValue, nValue*2, nValue*3, 200);
           glTexCoord2f(0.0f, 0.0f); glVertex3f(-45.0f,-5.0f,-65.0f); // Bottom Left Of The Texture And Quad
           glTexCoord2f(1.0f, 0.0f); glVertex3f( 45.0f,-5.0f,-65.0f); // Bottom Right Of The Texture And Quad
           glTexCoord2f(1.0f, 1.0f); glVertex3f( 45.0f, 45.0f,-65.0f); // Top Right Of The Texture And Quad
           glTexCoord2f(0.0f, 1.0f); glVertex3f(-45.0f, 45.0f,-65.0f); // Top Left Of The Texture And Quad
        glEnd();


        GL_LookAt(0.0f, 0.0f, 15.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f);

        glEnable(GL_BLEND);
        glDepthMask(GL_FALSE); // <------------

        glRotatef(PulseAngle, 0.0f, 1.0f, 0.0f);

        if (ShowLogo == 0) {
            glEnable (GL_LIGHTING);
            glBindTexture(GL_TEXTURE_2D, mt[1].Texture);

            for (K = 0; K < 128; K += 4) {
                long ColorRange = (long) (32 - (64 - (pv[K] * 30 + 30)));
                // Draw burning cylinder
                glPushMatrix();
                    long IncreaseRange = 0; if (ColorRange > 6) { IncreaseRange = 12; }
                    c = min(ColorRange + IncreaseRange, 31);
                    glColor4ub(gnR[c], gnG[c], gnB[c], 255);
                    glTranslatef(-UseSize + UseSize * K / 64.0f, -1.5f, 0.0f);
glRotatef(-PulseAngle, 0.0f, 1.0f, 0.0f);
                    glRotatef(-90.0f, 1.0f, 0.0f, 0.0f);
                    DrawCylinder(0.75f, pv[K] * 8.0f, 1.0f, 0.875f);
                glPopMatrix();
            
                // Draw ground reflection
                c = min(ColorRange, 31);
                glPushMatrix();
                    glColor4ub(gnR[c], gnG[c], gnB[c], 255);
                    glTranslatef(-UseSize + UseSize * K / 64.0f, -2.0f, 0.0f);
                    glRotatef(90.0f, 1.0f, 0.0f, 0.0f);
                    float ax = 3.0f; 
                    float ay = pv[K] * 8;
                    glBegin(GL_QUADS);
                        glTexCoord2f(0.0f, 0.0f); glVertex3f(-ax,-ay, 0.0f);
                        glTexCoord2f(1.0f, 0.0f); glVertex3f( ax,-ay, 0.0f);
                        glTexCoord2f(1.0f, 1.0f); glVertex3f( ax, ay, 0.0f);
                        glTexCoord2f(0.0f, 1.0f); glVertex3f(-ax, ay, 0.0f);
                    glEnd();
                glPopMatrix();
           
            }
        }

        glDepthMask(GL_TRUE); // <------------
        glDisable(GL_BLEND);

        glFlush();

        // Very important we must reassign BBP.RC to the new BBP.DC
        // Note; don//t use permanent DC, this produce better and smoother display
        wglMakeCurrent(BBP.dc, BBP.rc);
        break;

    case BBP_CREATE:
        // Retrieve plugin details
        strcpy(BBP.title, "Fahrenheit 451");
        strcpy(BBP.author, "Patrice Terrier");
        BBP.version  = MAKEWORD(1, 0); // Version 1.0"
        BBP.renderto = BBP_OPENGL;     // or BBP_GDIPLUS, or BBP_DIRECTX
        break;

    case BBP_INIT:
        // Do your code initialisation there
        bbp_ColorInit();

        glClearColor((GetRValue(BBP.backargb) + 1) / 256.0f, (GetGValue(BBP.backargb) + 1) / 256.0f, (GetBValue(BBP.backargb) + 1) / 256.0f, 1.0f);

        glEnable(GL_DEPTH_TEST);
        glShadeModel(GL_SMOOTH);
        glEnable (GL_LIGHTING);
        glEnable (GL_LIGHT0);
        glEnable(GL_COLOR_MATERIAL);
        glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
        glEnable(GL_NORMALIZE);
        glBlendFunc(GL_ONE, GL_ONE);

        glEnable(GL_ALPHA_TEST);

        glEnable(GL_TEXTURE_2D);
        Path_Combine(mt[0].FullName, TexturePath(), L"superbeam.jpg"); mt[0].ID = 1; mt[0].Square = 1;
        Path_Combine(mt[1].FullName, TexturePath(), L"beam.jpg"); mt[1].ID = 2; mt[1].Square = 0;
        MakeMultipleTexture(&mt[0], 2);

        MakeColorLUT();
        break;

    case BBP_SIZE:
        // The size of the view port has changed.
        RECT rc; GetClientRect(BBP.parent, &rc);
        glViewport(0, 0, Width(rc), Height(rc));

        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        rAspect = 0.0f;
        if (Height(rc)) { rAspect = (float) Width(rc) / (float) Height(rc); }
        GL_Perspective(35.0f, rAspect, 0.1f, 150.0f);

        glMatrixMode(GL_MODELVIEW);
        break;

    case BBP_DESTROY:
        // Free up your resources there
        DestroyTexture (&mt[0], 2);
        break;

    default:
        nRet = BBP_ERROR;
        break;
    }
    return nRet;
}

BOOL WINAPI DllMain (HINSTANCE hinstDLL, IN DWORD fdwReason, IN LPVOID lpvReserved) {
    switch (fdwReason) {
    case DLL_PROCESS_ATTACH:
        break;
    case DLL_PROCESS_DETACH:
        break;
    case DLL_THREAD_ATTACH:
        break;
    case DLL_THREAD_DETACH:
        break;
    }
    return TRUE;
}
