// CArray.cpp
#ifndef CArray_cpp
#define CArray_cpp
#define NEW new

template <class datatype> class CArray            // This entity is a templated class for creating dynamic
{                                                 // multi-dimensional arrays of from one to four dimensions.
 public:                                          // It allows for a basic language type syntax for doing
 CArray() : pObjs(0)                              // the above.  GNU C++ compilers implement dynamic array
 {                                                // allocation but MS VC++ compilers don't.  Since I wanted
  this->iNumObjects=0;                            // to test compile with both I developed this class.
  d1=d2=d3=d4=0;
 }

 CArray(size_t i) : pObjs(0)                                  // One Dimensional Array Constructor
 {
  this->iNumObjects=i;
  this->pObjs = NEW datatype[this->iNumObjects]();
  d1=i, d2=0, d3=0, d4=0;
 }

 CArray(size_t i, size_t j) : pObjs(0)                        // Two Dimensional Array Constructor
 {
  this->iNumObjects=i*j;
  this->pObjs = NEW datatype[this->iNumObjects]();
  d1=i, d2=j, d3=0, d4=0;
 }

 CArray(size_t i, size_t j, size_t k) : pObjs(0)              // Three Dimensional Array Constructor
 {
  this->iNumObjects=i*j*k;
  this->pObjs = NEW datatype[this->iNumObjects]();
  d1=i, d2=j, d3=k, d4=0;
 }

 CArray(size_t i, size_t j, size_t k, size_t l) : pObjs(0)    // Four Dimensional Array Constructor
 {
  this->iNumObjects=i*j*k*l;
  this->pObjs = NEW datatype[this->iNumObjects]();
  d1=i, d2=j, d3=k, d4=l;
 }
 
 bool Dim(size_t i)
 {
  if(this->pObjs)
     delete [] this->pObjs;
  this->iNumObjects=i;
  this->pObjs=NEW datatype[this->iNumObjects]();
  d1=i, d2=0, d3=0, d4=0;
  if(this->pObjs)
     return true;
  else
     return false;
 }
 
 bool Redim(size_t i, bool blnPreserve)
 {
  datatype* pObj=NULL;
  
  if(this->pObjs)
  {    
     pObj=NEW datatype[i]();
     if(pObj)
     {
        if(blnPreserve)
        {   
           size_t iObs=0;
           if(i<this->iNumObjects)
              iObs=i;
           else          
              iObs=this->iNumObjects;
           for(size_t i=0; i<iObs; i++)
               pObj[i]=this->pObjs[i];
        }
        delete [] this->pObjs;
        this->pObjs=pObj;
        this->iNumObjects=i;
        d1=i;
        return true;
     }
     else
        return false; 
  }  
  
  return false;  
 } 
 
 bool Dim(size_t i, size_t j)
 {
  if(this->pObjs)
     delete [] this->pObjs;
  this->iNumObjects=i*j;
  this->pObjs=NEW datatype[this->iNumObjects]();
  d1=i, d2=j, d3=0, d4=0;
  if(this->pObjs)
     return true;
  else
     return false;
 }

 bool Dim(size_t i, size_t j, size_t k)
 {
  if(this->pObjs)
     delete [] this->pObjs;
  this->iNumObjects=i*j*k;
  this->pObjs=NEW datatype[this->iNumObjects]();
  d1=i, d2=j, d3=k, d4=0;
  if(this->pObjs)
     return true;
  else
     return false;
 }

 bool Dim(size_t i, size_t j, size_t k, size_t l)
 {
  if(this->pObjs)
     delete [] this->pObjs;
  this->iNumObjects=i*j*k*l;
  this->pObjs=NEW datatype[this->iNumObjects]();
  d1=i, d2=j, d3=k, d4=l;
  if(this->pObjs)
     return true;
  else
     return false;
 }

 datatype& operator()(size_t i)                               // One Dimensional Accessor
 {
  return pObjs[i];
 }

 datatype& operator()(size_t i, size_t j)                     // Two Dimensional Accessor
 {
  return pObjs[i*d2 + j];
 }

 datatype& operator()(size_t i, size_t j, size_t k)           // Three Dimensional Accessor
 {
  return pObjs[i*d2 + j + k*d1*d2];
 }

 datatype& operator()(size_t i, size_t j, size_t k, size_t l) // Four Dimensional Accessor
 {
  return pObjs[i*d2 + j + k*d1*d2 + l*d1*d2*d3];
 }

 bool blnMemoryIsGood()
 {
  return !!pObjs;
 }

 int UBound(int iDim)
 {
  if(iDim==1)
     return d1-1;
  if(iDim==2)
     return d2-1;
  if(iDim==3)
     return d3-1;
  if(iDim==4)
     return d4-1;
  else
     return 0;
 }
datatype* getptr()
{
	return pObjs;
}	

 ~CArray()
 {
  if(this->pObjs)
     delete [] this->pObjs;
 }

 private:
 
  datatype* pObjs;       // pointer to the base memory allocation for array
 size_t       iNumObjects; // We'll need this to zero memory for non-class types
 size_t       d1;          // Dimension #1
 size_t       d2;          // Dimension #2
 size_t       d3;          // Dimension #3
 size_t       d4;          // Dimension #4
};

#endif
