#pragma once

#define long_proc typedef long (__stdcall *zProc)
#define ubound(arr) sizeof(arr) / sizeof(arr[0])
#define strSize(s) sizeof(s) / sizeof(s[0])
#define AddStr(s1, s2) (wcsncat_s(s1, s2, _TRUNCATE))
#define DegreesToRadians(Deg) ((Deg) * 0.0174532925199432957)
#define RadiansToDegrees(Rad) ((Rad) * 57.29577951308232)

const int BBP_RENDER      = 1;       // Render the scene.
const int BBP_CREATE      = 2;       // Retrieve Title, Name, Version, Render mode.
const int BBP_INIT        = 3;       // Init the OpenGL.
const int BBP_SIZE        = 4;       // The size of the control has changed.
const int BBP_KEYBOARD    = 5;       // All keyborad message.
const int BBP_MOUSE       = 6;       // All mouse messages.
const int BBP_DESTROY     = 7;       // Free Up resources.

const int BBP_GDIPLUS     = 0;       // GDImage GDIPLUS compatible mode.
const int BBP_OPENGL      = 1;       // OpenGL mode.
const int BBP_DIRECTX     = 2;       // DirectX mode (for future extension).

const int BBP_SUCCESS     = 0;
const int BBP_ERROR       = -1;

struct BBPLUGIN {
    UINT msg;                   // The plugin's message (see above constant list).
    HWND parent;                // The parent window handle.
    HDC dc;                     // The parent window DC (while in play mode).
    HGLRC rc;                   // The parent OpenGL RC (while in play mode).
    WORD lpeak;                 // The left audio channel peak value (while in play mode).
    WORD rpeak;                 // The right audio channel peak value (while in play mode).
    char title[32];             // Plugin's name or title.
    char author[64];            // Plugin's author name.
    DWORD version;              // LOWRD major, HIWRD minor.
    long renderto;              // BBP_GDIPLUS, BBP_OPENGL, BBP_DIRECTX.
    long backargb;              // Default ARGB color background.
    float* fftdata;             // dword pointer to the FFT() as single array.
    WORD fftsize;               // Size of the FFT array.

    UINT winmsg;                // True Windows message.
    WPARAM wparam;              // wParam
    LPARAM lparam;              // lParam

    short* wimdata;             // dword pointer to the wave MM_WIM_DATA.
    QWORD medialength;          // Media length.
    QWORD mediapos;             // Media pos.

    char reserved[50];          // Reserved for future extension.
} ;                             // Structure size = 256

struct BBPTEXTURE {
    long  ID;
    WCHAR FullName[MAX_PATH];
    long  Texture;
    long  Square;  // boolean TRUE or FALSE value 
    long  alpha;
};

struct COLORBYTES {
    BYTE B;
    BYTE G;
    BYTE R;
    BYTE A;
};

long zTrace(IN WCHAR* sPtr) {
    long nRet = 0;
    static HMODULE hDll;
    if (hDll == 0) {
        if (sizeof(LONG_PTR) == 8) {
            hDll = LoadLibrary(L"zTrace64");
        }
        else {
            hDll = LoadLibrary(L"zTrace32");
        }
    }
    if (hDll) {
        long_proc(WCHAR*);
        static zProc hProc;
        if (hProc == 0) { hProc = (zProc)GetProcAddress(hDll, "zTrace"); }
        if (hProc) { nRet = hProc(sPtr); }
    }
    return nRet;
}

long bbpARGB (IN BYTE A, IN BYTE R, IN BYTE G, IN BYTE B) {
    COLORBYTES bytestruct;
    long result = 0;
    bytestruct.A = A;
    bytestruct.R = R;
    bytestruct.G = G;
    bytestruct.B = B;
    CopyMemory(&result, &bytestruct, sizeof(COLORBYTES));
    return result;
}

long Width(IN RECT &r){
    return r.right - r.left;
}

long Height(IN RECT &r) {
    return r.bottom - r.top;
}

//WCHAR* STRL(IN long N) {
//    static WCHAR longstr[33] = {0};
//    _ltow(N, longstr, 10);
//    return (WCHAR*)longstr;
//}

WCHAR* STRF(IN float N) {
    static WCHAR ws[33] = {0};
    swprintf(ws, L"%f", N);
    return (WCHAR*) ws;
}

void DestroyTexture(IN BBPTEXTURE* tp, IN long mtCount) {
    long K, OkDelete = 0;
    if (mtCount) {
        long* Texture = new long[mtCount]; memset(Texture, 0, mtCount);
        for (K = 0; K < mtCount; K++) {
            Texture[K] = tp[K].Texture; if (Texture[K]) { OkDelete = -1; }
        }
        if (OkDelete) { glDeleteTextures(mtCount, (GLuint*)&Texture[0]); }
        delete[] Texture;
    }
}

HMODULE gdiplib() {
    static HMODULE hGdip;
    if (hGdip == 0) { hGdip = LoadLibrary(L"GDIPLUS"); }
    return hGdip;
}

long GdipLoadImageFromFile (IN WCHAR* sImgName, OUT LONG_PTR &lpImg) {
    long nRet = -1; // Error
    HMODULE hModule = gdiplib();
    lpImg = 0;
    if (hModule) {
        //WCHAR* wc = (WCHAR*)sImgName.c_str();
        long_proc (WCHAR*, LONG_PTR*);
        static zProc hProc;
        if (hProc == 0) { hProc = (zProc) GetProcAddress(hModule, "GdipLoadImageFromFile"); }
        if (hProc) { 
            nRet = hProc(sImgName, &lpImg); 
        }
    }
    return nRet;
}

long GdipGetImageWidth(IN LONG_PTR lpImg, OUT long &imgW) {
    long nRet = -1; // Error
    HMODULE hModule = gdiplib();
    imgW = 0;
    if (hModule) {
        long_proc (LONG_PTR, long*);
        static zProc hProc;
        if (hProc == 0) { hProc = (zProc) GetProcAddress(hModule, "GdipGetImageWidth"); }
        if (hProc) { nRet = hProc(lpImg, &imgW); }
    }
    return nRet;
}

long GdipGetImageHeight(IN LONG_PTR lpImg, OUT long &imgH) {
    long nRet = -1; // Error
    HMODULE hModule = gdiplib();
    imgH = 0;
    if (hModule) {
        long_proc (LONG_PTR, long*);
        static zProc hProc;
        if (hProc == 0) { hProc = (zProc) GetProcAddress(hModule, "GdipGetImageHeight"); }
        if (hProc) { nRet = hProc(lpImg, &imgH); }
    }
    return nRet;
}

long GdipCreateFromHDC(IN HDC hDC, OUT LONG_PTR &graphics) {
    long nRet = -1; // Error
    HMODULE hModule = gdiplib();
    graphics = 0;
    if (hModule) {
        long_proc (HDC, LONG_PTR*);
        static zProc hProc;
        if (hProc == 0) { hProc = (zProc) GetProcAddress(hModule, "GdipCreateFromHDC"); }
        if (hProc) { nRet = hProc(hDC, &graphics); }
    }
    return nRet;
}

long GdipSetInterpolationMode(IN LONG_PTR graphics, IN long interpolation) {
    long nRet = -1; // Error
    HMODULE hModule = gdiplib();
    if (hModule) {
        long_proc (LONG_PTR, long);
        static zProc hProc;
        if (hProc == 0) { hProc = (zProc) GetProcAddress(hModule, "GdipSetInterpolationMode"); }
        if (hProc) { nRet = hProc(graphics, interpolation); }
    }
    return nRet;
}

long GdipBitmapGetPixel(IN LONG_PTR lpImg, IN long x, IN long y, OUT long &ARGB) {
    long nRet = -1; // Error
    HMODULE hModule = gdiplib();
    ARGB = 0;
    if (hModule) {
        long_proc (LONG_PTR, long, long, long*);
        static zProc hProc;
        if (hProc == 0) { hProc = (zProc) GetProcAddress(hModule, "GdipBitmapGetPixel"); }
        if (hProc) { nRet = hProc(lpImg, x, y, &ARGB); }
    }
    return nRet;
}

long GdipDrawImageRectRectI(IN LONG_PTR graphics, IN LONG_PTR lpImg, IN long dstX, IN long dstY, IN long dstW, IN long dstH, IN long srcX, IN long srcY, IN long srcW, IN long srcH, IN long srcUnit, IN LONG_PTR imageattr, IN LONG_PTR lpCallback, IN LONG_PTR callbackdata) {
    long nRet = -1; // Error
    HMODULE hModule = gdiplib();
    if (hModule) {
        long_proc (LONG_PTR, LONG_PTR, long, long, long, long, long, long, long, long, long, LONG_PTR, LONG_PTR, LONG_PTR);
        static zProc hProc;
        if (hProc == 0) { hProc = (zProc) GetProcAddress(hModule, "GdipDrawImageRectRectI"); }
        if (hProc) { nRet = hProc(graphics, lpImg, dstX, dstY, dstW, dstH, srcX, srcY, srcW, srcH, srcUnit, imageattr, lpCallback, callbackdata); }
    }
    return nRet;
}

long GdipDeleteGraphics(IN LONG_PTR graphics) {
    long nRet = -1; // Error
    HMODULE hModule = gdiplib();
    if (hModule) {
        long_proc (LONG_PTR);
        static zProc hProc;
        if (hProc == 0) { hProc = (zProc) GetProcAddress(hModule, "GdipDeleteGraphics"); }
        if (hProc) { nRet = hProc(graphics); }
    }
    return nRet;
}

long GdipDisposeImage(IN LONG_PTR lpImg) {
    long nRet = -1; // Error
    HMODULE hModule = gdiplib();
    if (hModule) {
        long_proc (LONG_PTR);
        static zProc hProc;
        if (hProc == 0) { hProc = (zProc) GetProcAddress(hModule, "GdipDisposeImage"); }
        if (hProc) { nRet = hProc(lpImg); }
    }
    return nRet;
}

long TexAlpha(IN long nAlpha, IN long RW) {
    static long WasAlpha;
    if (RW) { WasAlpha = nAlpha; }
    return WasAlpha;
}

//long IsFileImage(IN WCHAR* zFileName) {
//    long nRet = 0;
//    WCHAR drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME], ext[_MAX_EXT];
//    if (zFileName) {
//        _wsplitpath_s(zFileName, drive, dir, fname, ext);
//        if (wcslen(ext)) {
//            wcsncat_s(ext, L".", 1);
//            CharLower(ext);
//            if (wcsstr(L".bmp.dib.gif.ico.jpg.jpeg.png.tif.tiff.", ext)) { nRet = -1; }
//        }
//    }
//    return nRet;
//}

BOOL FileExist(IN WCHAR* szFileSpec) {
    WIN32_FIND_DATA fd = {0};
    BOOL bRet = FALSE;
    if (wcslen(szFileSpec)) {
        HANDLE hFind;
        hFind = FindFirstFileW(szFileSpec, &fd);
        if (hFind != INVALID_HANDLE_VALUE) {
            FindClose(hFind);
            bRet = TRUE;
        }
    }
    return bRet;
}

WCHAR* EXEpath() {
    static WCHAR zPath[MAX_PATH] = { 0 };
    if (wcslen(zPath) == 0) {
        if (GetModuleFileName(0, zPath, sizeof(zPath))) {
            long nLen = (long)wcslen(zPath);
            long nPos = nLen;
            for (long K = 1; K <= nLen; K++) {
                if (_wcsicmp(&zPath[nPos - K], L"\\") != 0) {
                    zPath[nPos - K] = NULL;
                } else { 
                    break;
                }
            }
        }
    }
    return zPath;
}

void Path_Combine(OUT WCHAR* zResource, IN WCHAR* path, IN WCHAR* combine) {
    long offset = (long) wcslen(path) * sizeof(WCHAR);
    ZeroMemory(zResource, MAX_PATH);
    MoveMemory(zResource, path, offset);
    MoveMemory(&zResource[offset / 2], &combine[0], (long) wcslen(combine) * sizeof(WCHAR));
}

WCHAR* TexturePath() {
    static WCHAR zResource[MAX_PATH] = { 0 };
    if (wcslen(zResource) == 0) { 
        Path_Combine(zResource, EXEpath(), L"BBplugin\\Texture\\");
    }
    return zResource;
}

long CreateGLTextureFromFileEX(IN WCHAR* ImgName, OUT long &Xin, OUT long &Yin, OUT BYTE* &lpArray, IN long SquareTexture) {
    long nRet = 0, nSwapRB = -1; // 12-10-2015: TRUE; swap red and blue bytes on default
    long K, xP, yP, xS, yS;
    LONG_PTR graphics = 0, imgAttr = 0, lpCallback = 0, callbackdata = 0;
    BYTE *pixels = NULL;

    long nAlphaFlag = 0;

    Xin = 0; Yin = 0;
    long WasSquareTexture = SquareTexture;

    // Make sure that a full path is being used.
    if (FileExist(ImgName)) {
        //if (IsFileImage(ImgName)) {
            // Load the image
            LONG_PTR img = NULL;

            nRet = GdipLoadImageFromFile(ImgName, img);

            if (nRet) { // ML: failure
                if (pixels) { delete[] pixels; }
                nRet = 0; // PAT: 12-18-2015 in this function nRet = -1 means success (not error), nRet = 0 means couldn't create the texture
                return nRet;
            }

            // Get the image height and width
            long imgW, imgH;
            GdipGetImageWidth(img, imgW);
            GdipGetImageHeight(img, imgH);
            if (SquareTexture > 2) { // (3 = texture of any size, 4 = MipMapping)
                xP = 0; yP = 0;
                xS = imgW; Xin = imgW;
                yS = imgH; Yin = imgH;
            } else {
                long P[11] = { 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048 };
                for (K = 10; K > -1; --K) {
                    if ((Xin == 0) && (imgW > P[K] - 1)) { Xin = P[K]; }
                    if ((Yin == 0) && (imgH > P[K] - 1)) { Yin = P[K]; }
                }
                if ((SquareTexture) && (Xin != Yin)) {
                    Xin = max(Xin, Yin); Yin = Xin;
                } else {
                    SquareTexture = 0;
                }
                // Perform scale adjustment (zIconise(imgW, imgH, Xin, Yin, xP, yP, xS, yS))
                float scale = 0.0f;
                if (imgW) { scale = (float) (Xin / (float)imgW); }
                if (scale > 1) { scale = 1; }
                xS = (long) (imgW * scale); yS = (long) (imgH * scale);
                // In case Height > Yin compute new scale factor
                if (yS > Yin) {
                    if (imgH) { scale = (float) (Yin / (float)imgH); }
                    xS = (long) (imgW * scale); yS = (long) (imgH * scale);
                }
                xP = (long) ((Xin - xS) * 0.5f); yP = (long) ((Yin - yS) * 0.5f);

                if (WasSquareTexture == 2) { xP = 0; yP = 0; xS = Xin; yS = Yin; } // Stretch the texture to the size of the memory bitmap target
            }

            HDC hIC = CreateICA("DISPLAY", NULL, NULL, NULL);
            HDC hDC = CreateCompatibleDC(hIC);
            BITMAPINFO bi;
            bi.bmiHeader.biSize = sizeof(bi.bmiHeader);
            bi.bmiHeader.biWidth = Xin;
            bi.bmiHeader.biHeight = Yin;
            bi.bmiHeader.biPlanes = 1;
            bi.bmiHeader.biBitCount = 32;
            bi.bmiHeader.biCompression = BI_RGB;
            HBITMAP hDIB = CreateDIBSection(hDC, &bi, 0, 0, 0, 0);

            SelectObject(hDC, hDIB);
            BYTE* pBits;
            // Draw image
            if (GdipCreateFromHDC(hDC, graphics) == 0) {
               // Set up stretching QualityMode
               GdipSetInterpolationMode(graphics, 2);
               // Fill the background with color of pixel(0,0)
               // if (SquareTexture) { // 04-09-2014
               if ((SquareTexture > 0) && (SquareTexture < 3)) {
                    long ARGBcolor;
                    GdipBitmapGetPixel(img, 0, 0, ARGBcolor);
                    BYTE A, R, G, B;
                    //zSplitColorARGB(ARGBcolor, A, R, G, B);
                    R = (BYTE)(ARGBcolor); // 12-21-2015 ML: could be a tad faster than zSplit, no?
                    G = (BYTE)(ARGBcolor >> 8);
                    B = (BYTE)(ARGBcolor >> 16);
                    A = (BYTE)(ARGBcolor >> 24);
                    BITMAP bm;
                    GetObject(hDIB, sizeof(bm), &bm);
                    pBits = (BYTE*) bm.bmBits;
                    for (K = 0; K < (Xin * Yin); ++K) {
                        pBits[3] = A;
                        pBits[2] = R;
                        pBits[1] = G;
                        pBits[0] = B;
                        pBits += 4;
                    }
                }

                // Draw image
                LONG_PTR imgAttr = 0;
                if (GdipDrawImageRectRectI(graphics, img, xP, yP, xS, yS, 0, 0, imgW, imgH, 2, imgAttr, lpCallback, callbackdata) == 0) {
                    bi.bmiHeader.biSize = sizeof(bi.bmiHeader);
                    bi.bmiHeader.biWidth       = Xin;
                    bi.bmiHeader.biHeight      = Yin;
                    bi.bmiHeader.biPlanes      = 1;
                    bi.bmiHeader.biBitCount    = 32;
                    bi.bmiHeader.biCompression = BI_RGB;

                    lpArray = (BYTE*) malloc(Xin * Yin * 4);
                    if (GetDIBits(hDC, hDIB, 0, Yin, &lpArray[0], &bi, 0)) {

                        // Perform Red and Green permutation
                        pBits = &lpArray[0]; BYTE R;
                        for (K = 0; K < (Xin * Yin); ++K) {

                            // 05-10-2015 to detect if we are using an alpha channel
                            if (nAlphaFlag == 0) {
                                if ((pBits[3] < 255) && (pBits[3] > 0)) {
                                    nAlphaFlag = -1; TexAlpha(nAlphaFlag, 1);
                                }
                            }

                            R = pBits[2];

                            // Texture mapping to allow correct mouse detection
                            if (R == 0) {
                                if (pBits[0] == 0) {
                                    if (pBits[1] == 0) { R = 1; }
                                }
                            }

                            if (nSwapRB) { // 12-21-2015 ML: FALSE can happen with uncompressed DDS
                                pBits[2] = pBits[0]; pBits[0] = R;
                            }

                            if (nAlphaFlag) { // ML: 12-01-2015
                                pBits[2] = (pBits[2] * pBits[3]) >> 8;
                                pBits[1] = (pBits[1] * pBits[3]) >> 8;
                                pBits[0] = (pBits[0] * pBits[3]) >> 8;
                            }

                            pBits += 4;
                        }
                        nRet = -1;
                    }
                }
                // Cleanup
                GdipDeleteGraphics(graphics);
            }
            DeleteObject(hDIB);
            if (hDC) { DeleteDC(hDC); }
            if (hIC) { DeleteDC(hIC); }

            GdipDisposeImage(img); img = 0;
            if (pixels) { delete[] pixels; }
        //}
    }
    return nRet;
}

long GL_ActiveTexture(IN long nTexture, IN long RW) {
    static long wasTexture;
    if (RW) { wasTexture = nTexture; }
    return wasTexture;
}

long GL_GetActiveTexture () { // dllexport
    return GL_ActiveTexture(0,0);
}

void GL_BindTexture (IN long nTextureMode, IN long nTextureToUse) { // dllexport
    glBindTexture(nTextureMode, nTextureToUse);
    GL_ActiveTexture(nTextureToUse, 1);
}

void MakeMultipleTexture (IN BBPTEXTURE* mt, IN long mtCount) {
    if (mtCount> 0) {
       long OkDelete = 0;
       long* Texture = new long[mtCount];
       memset(Texture, 0, sizeof(long) * mtCount);
       for (long K = 0; K < mtCount; K++) {
           Texture[K] = mt[K].Texture; if (Texture[K]) { OkDelete = -1; }
       }
       if (OkDelete) { glDeleteTextures(mtCount, (GLuint*) &Texture[0]); }
       glGenTextures(mtCount, (GLuint*) &Texture[0]);
       long nRet = glGetError();
       if (nRet == 0) {
           for (long K = 0; K < mtCount; K++) {
               BYTE* lpArray = NULL;
               long xSize = 0, ySize = 0;

               TexAlpha(0, 1); // 05-10-2015 to detect if we are using an alpha channel

               if (CreateGLTextureFromFileEX(mt[K].FullName, xSize, ySize, lpArray, mt[K].Square)) {
                   mt[K].Texture = Texture[K];
                   mt[K].alpha = TexAlpha(0, 0); // 05-10-2015 to detect if we are using an alpha channel
                   GL_BindTexture(GL_TEXTURE_2D, Texture[K]); nRet = glGetError();
                   if (nRet == 0) {
                       if (mt[K].Square == 4) { // MipMapping in ZI_CreateGLTextureFromFileEX
                           glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR);
                           glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
                           glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
                           glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
                           gluBuild2DMipmaps(GL_TEXTURE_2D, 4, xSize, ySize, GL_RGBA, GL_UNSIGNED_BYTE, lpArray);
                       } else {
                           glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
                           glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
                           glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE); // 01-05-2016 ML: to avoid wallpaper bottom line artifacts
                           glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE); // ditto
                           glTexImage2D(GL_TEXTURE_2D, 0, 4, xSize, ySize, 0, GL_RGBA, GL_UNSIGNED_BYTE, lpArray);
                       }
                       nRet = glGetError();
                   }
                   free (lpArray);
               }

           }
       }
       delete [] Texture;
    }
}

void GL_LookAt (IN float eyeX, IN float eyeY, IN float eyeZ, IN float lookAtX, IN float lookAtY, IN float lookAtZ, IN float upX, IN float upY, IN float upZ) { // dllexport
    // i am not using here proper implementation for vectors.
    // if you want, you can replace the arrays with your own 
    // vector types

    // calculating the viewing vector
    float f[3];
    f[0] = lookAtX - eyeX;
    f[1] = lookAtY - eyeY;
    f[2] = lookAtZ - eyeZ;

    float fMag = (float)sqrt(f[0]*f[0] + f[1]*f[1] + f[2]*f[2]);
    float upMag = (float)sqrt(upX*upX + upY*upY + upZ*upZ);

    // normalizing the viewing vector
    if (fMag) {
        f[0] /= fMag;
        f[1] /= fMag;
        f[2] /= fMag;
    }

    // normalising the up vector. no need for this here if you have your 
    // up vector already normalised, which is mostly the case.
    if (upMag) {
        upX /= upMag;
        upY /= upMag;
        upZ /= upMag;
    }

    float s[3];
    s[0] = f[1] * upZ  - upY  * f[2];
    s[1] = upX  * f[2] - f[0] * upZ;
    s[2] = f[0] * upY  - upX  * f[1];
    float u[3];
    u[0] = s[1] * f[2] - f[1] * s[2];
    u[1] = f[0] * s[2] - s[0] * f[2];
    u[2] = s[0] * f[1] - f[0] * s[1];

    float M[16] = { s[0], u[0], -f[0], 0,
                    s[1], u[1], -f[1], 0,
                    s[2], u[2], -f[2], 0, 
                    0, 0, 0, 1 };

    glMultMatrixf(&M[0]);
    glTranslatef (-eyeX, -eyeY, -eyeZ);
}

void GL_Perspective(IN double fovY, IN double aspect, IN double zNear, IN double zFar) {
    const GLdouble Pi = 3.1415926535897932384626433832795;
    GLdouble fW, fH;
    fH = tan( fovY / 360 * Pi ) * zNear;
    fW = fH * aspect;
    glFrustum(-fW, fW, -fH, fH, zNear, zFar);
}

void DrawQuad(IN float x, IN float y, IN float z, IN long nTexture) {
   glBindTexture(GL_TEXTURE_2D, nTexture);
   glBegin(GL_QUADS);
       glTexCoord2f(0.0f, 0.0f); glVertex3f(-x,-y, z); // Bottom Left Of The Texture And Quad
       glTexCoord2f(1.0f, 0.0f); glVertex3f( x,-y, z); // Bottom Right Of The Texture And Quad
       glTexCoord2f(1.0f, 1.0f); glVertex3f( x, y, z); // Top Right Of The Texture And Quad
       glTexCoord2f(0.0f, 1.0f); glVertex3f(-x, y, z); // Top Left Of The Texture And Quad
   glEnd();
}

int random() {
    static unsigned int next = 1;
    next = next * 1103515245 + 12345;
    return (unsigned int)(next>>16) & RAND_MAX;
}

size_t rnd(IN long nMin, IN long nMax) {
 double dblRange = nMax - nMin;
 double dblMaxFactor = dblRange / 32767.0;
 double dblRandomNumber = (double) random();

 return (size_t) (nMin + dblMaxFactor * dblRandomNumber);
}

void RandomizeTimer() {
//    //static long done;
//    //if (!done) { srand((DWORD) time(NULL)); done = !done; }
//    srand((DWORD) time(NULL));
}

float rsRandf(IN float x) {
    return x * (float(random()) / RAND_MAX);
}

long rsRandi(IN long x) {
    return random() % x;
}
