//+--------------------------------------------------------------------------+
//|                                                                          |
//|                                LaserBeam                                 |
//|                          BassBox OpenGL plugin                           |
//|                                                                          |
//|                              Version 2.00                                |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|                                                                          |
//|                         copyright(c) 2007-2017                           |
//|                                                                          |
//|                Patrice Terrier http://www.zapsolution.com                |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                  Project started on : 10-30-2007 (MM-DD-YYYY)            |
//|                        Last revised : 02-08-2017 (MM-DD-YYYY)            |
//+--------------------------------------------------------------------------+

#include <windows.h>
//#include "..\TCLib\stdio.h"
//#include "..\TCLib\string.h"
//#include "..\TCLib\stdlib.h"
//#include "..\TCLib\memory.h"
//#include "..\TCLib\malloc.h"
#include "..\TCLib\math.h"
//#include "..\TCLib\tchar.h"
#include "..\TCLib\Strings.cpp"

#include <gl/GL.h>
#include <gl/GLU.h>
#include "gl2.h"

typedef ULONGLONG QWORD;
#include "BBplugin.h"
#define ExportC extern "C" _declspec (dllexport)

long g_color[34];
float gsR[255], gsG[255], gsB[255];

void bbp_ColorInit() {
    g_color[0]  = 0;
    g_color[1]  = bbpARGB(255, 32,32,32);
    g_color[2]  = bbpARGB(255, 0,44,233);
    g_color[3]  = bbpARGB(255, 0,67,210);
    g_color[4]  = bbpARGB(255, 0,89,187);
    g_color[5]  = bbpARGB(255, 0,112,164);
    g_color[6]  = bbpARGB(255, 0,135,142);
    g_color[7]  = bbpARGB(255, 0,159,117);
    g_color[8]  = bbpARGB(255, 0,183,88);
    g_color[9]  = bbpARGB(255, 0,207,58);
    g_color[10] = bbpARGB(255, 0,231,29);
    g_color[11] = bbpARGB(255, 26,234,26);
    g_color[12] = bbpARGB(255, 52,237,23);
    g_color[13] = bbpARGB(255, 79,240,20);
    g_color[14] = bbpARGB(255, 105,243,17);
    g_color[15] = bbpARGB(255, 126,245,14);
    g_color[16] = bbpARGB(255, 147,248,11);
    g_color[17] = bbpARGB(255, 168,250,8);
    g_color[18] = bbpARGB(255, 189,253,5);
    g_color[19] = bbpARGB(255, 210,255,2);
    g_color[20] = bbpARGB(255, 233,255,0);
    g_color[21] = bbpARGB(255, 255,255,0);
    g_color[22] = bbpARGB(255, 255,251,0);
    g_color[23] = bbpARGB(255, 255,235,0);
    g_color[24] = bbpARGB(255, 255,215,0);
    g_color[25] = bbpARGB(255, 255,196,0);
    g_color[26] = bbpARGB(255, 255,176,0);
    g_color[27] = bbpARGB(255, 255,156,0);
    g_color[28] = bbpARGB(255, 253,137,0);
    g_color[29] = bbpARGB(255, 255,117,0);
    g_color[30] = bbpARGB(255, 255,97,0);
    g_color[31] = bbpARGB(255, 255,78,0);
    g_color[32] = bbpARGB(255, 255,58,0);
    g_color[33] = bbpARGB(255, 255,0,0);
}

long bbp_LevelColor(IN long nLevel) {
    long nColor = 0;
    nLevel += 1; if (nLevel > 33) { nLevel = 33; }
    if (nLevel > 0) { nColor = g_color[nLevel]; }
    return nColor;
}

void MakeColorLUT() {
    COLORBYTES bytestruct;
    long RGBColor = 0;
    for (long K = 0; K < 256; K++) {
        RGBColor = bbp_LevelColor(K);
        MoveMemory(&bytestruct, &RGBColor, sizeof(COLORBYTES));
        gsR[K] = (float)(bytestruct.R / 255.0f);
        gsG[K] = (float)(bytestruct.G / 255.0f);
        gsB[K] = (float)(bytestruct.B / 255.0f);
    }
}

ExportC long BBProc (OUT BBPLUGIN &BBP) {
    long nRet = BBP_SUCCESS;
    const float SmoothValue = 0.8f, Cx = -515.0f, Cy = -180.0f, Cz = -23.0f;
    float fft[256], SoundBuffer[256], rAspect;
    static BBPTEXTURE mt;

    nRet = BBP_SUCCESS;

    switch (BBP.msg) {
    case BBP_RENDER:
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

        glLoadIdentity();

        glTranslatef(0.0f, -5.0f, Cz);
        //glTranslatef(0.0f, -2.0f, -90.0f);//Cz);

        glRotatef(Cx, 1.0f, 0.0f, 0.0f);
        glRotatef(Cy, 0.0f, 1.0f, 0.0f);

        // This is where the magic happens:
        // BBP.fftdata returns a pointer to an area of memory containing an array of 256 singles.
        //float* fft = new float[BBP.fftsize];
        MoveMemory(&fft[0], BBP.fftdata, 256 * sizeof(float));
        //float* SoundBuffer = new float[BBP.fftsize];

        // Try and smooth out the buffer by averaging the last results:
        for (long K = 0; K < 256; K++) {
            // We use SQR to bring out the midtone of the music, otherwise the spectrum is very flat.
            SoundBuffer[K] = (float) sqrt(fft[K]) * SmoothValue;
        }

        for (long Direction = -1; Direction < 2; Direction++) {
            if (Direction) {
                for (long K = 0; K < 256; K++) {
                    glPushMatrix();
                    // The iterator K goes from 0 (deepest bass) to 255 (highest treble pitch)
                    // Ax is the distance from the center. Ay & By are width and height of the flares.
                    float Ax = K / 255.0f * 100.0f * Direction;
               
                    float Ay = 2 + SoundBuffer[K] * (50.0f + K / 255.0f * 70.0f);        // As K increases, the relative size of the signal
                    float By = 2 + SoundBuffer[K] * (50.0f + K / 255.0f * 70.0f) * 0.2f; // drops, so the (K / 255 * 20) compensates by boosting the signal at higher frequencies.

                    glColor4f(gsR[K] * SoundBuffer[K], gsG[K] * SoundBuffer[K], gsB[K] * SoundBuffer[K] + 0.1f, 0.7f * SmoothValue);
                    glTranslatef(Ax, 0.0f, 0.0f);
                    glRotatef(-Cx, 1.0f, 0.0f, 0.0f);
                    glRotatef(-Cy, 0.0f, 1.0f, 0.0f);

                    glBegin(GL_QUADS);
                         glTexCoord2f(0.0f,0.0f); glVertex3f(-By , Ay, 0.0f);
                         glTexCoord2f(1.0f,0.0f); glVertex3f( By , Ay, 0.0f);
                         glTexCoord2f(1.0f,1.0f); glVertex3f( By ,-Ay, 0.0f);
                         glTexCoord2f(0.0f,1.0f); glVertex3f(-By ,-Ay, 0.0f);
                    glEnd();
               
                    glPopMatrix();
                }
            }
        }

        // Very important we must reassign BBP.RC to the new BBP.DC
        // Note: don't use permanent DC, this produce better and smoother display
        wglMakeCurrent(BBP.dc, BBP.rc);
        break;

    case BBP_CREATE:
        // Retrieve plugin details
        strcpy(BBP.title, "Laser beam");
        strcpy(BBP.author, "Patrice Terrier");
        BBP.version  = MAKEWORD(1, 0); // Version 1.0"
        BBP.renderto = BBP_OPENGL;     // or BBP_GDIPLUS, or BBP_DIRECTX
        break;
         
    case BBP_INIT:
        // Do your code initialisation there
        bbp_ColorInit();

        glClearColor((GetRValue(BBP.backargb) + 1) / 256.0f, (GetGValue(BBP.backargb) + 1) / 256.0f, (GetBValue(BBP.backargb) + 1) / 256.0f, 1.0f);

        glShadeModel(GL_SMOOTH);      // Enables Smooth Color Shading
        glClearDepth((GLclampd) 1.0); // Depth Buffer Setup

        glDisable(GL_DEPTH_TEST); // Disable Depth Buffer

        glBlendFunc(GL_ONE, GL_ONE);

        glDepthFunc(GL_LESS);     // The Type Of Depth Test To Do

        glDisable(GL_LIGHTING);

        glEnable(GL_BLEND);
        glAlphaFunc(GL_GREATER, 0.01f);

        glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

        glEnable(GL_TEXTURE_2D);
        Path_Combine(mt.FullName, TexturePath(), L"Beam.jpg"); mt.ID = 1; mt.Square = 3; // Use mipmapping
        MakeMultipleTexture(&mt, 1);

        MakeColorLUT();
        break;

    case BBP_SIZE:
        // The size of the view port has changed.
        RECT rc; GetClientRect(BBP.parent, &rc);
        glViewport(0, 0, Width(rc), Height(rc));

        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        rAspect = 0.0f;
        if (Height(rc)) { rAspect = (float) Width(rc) / (float) Height(rc); }
        GL_Perspective(50.0f, rAspect, 0.01f, 5000.0f);

        glMatrixMode(GL_MODELVIEW);
        break;

    case BBP_DESTROY:
        // Free up your resources there
        DestroyTexture (&mt, 1);
        break;

    default:
        nRet = BBP_ERROR;
        break;
    }
    return nRet;
}

BOOL WINAPI DllMain (HINSTANCE hinstDLL, IN DWORD fdwReason, IN LPVOID lpvReserved) {
    switch (fdwReason) {
    case DLL_PROCESS_ATTACH:
        break;
    case DLL_PROCESS_DETACH:
        break;
    case DLL_THREAD_ATTACH:
        break;
    case DLL_THREAD_DETACH:
        break;
    }
    return TRUE;
}
