/******************************************************************************
Module name: Win95ADG.h
Written by: Jeffrey Richter and Jonathan Locke
Notices: Copyright (c) 1995 Jeffrey Richter
Purpose: Common header file for "Windows 95: A Developer's Guide"
         Contains handy macros and definitions used throughout
         the applications.
******************************************************************************/



// This macro evaluates to the number of elements in an array. 
#define adgARRAY_SIZE(Array) (sizeof(Array) / sizeof((Array)[0]))


/////////////////////////////// adgINRANGE Macro //////////////////////////////


// This macro evaluates to TRUE if val is between lo and hi inclusive.
#define adgINRANGE(lo, val, hi) (((lo) <= (val)) && ((val) <= (hi)))


//////////////////////////// Assert/Verify Macros /////////////////////////////


#define adgFAIL(szMSG) {                                                  \
      MessageBox(GetActiveWindow(), szMSG,                                \
         __TEXT("Assertion Failed"), MB_OK | MB_ICONERROR);               \
      DebugBreak();                                                       \
   }

// Put up an assertion failure message box
#define adgASSERTFAIL(file,line,expr) {                                   \
      TCHAR sz[128];                                                      \
      wsprintf(sz, __TEXT("File %hs, line %d : %hs"), file, line, expr);  \
      adgFAIL(sz);                                                        \
   }

// Put up a message box if an assertion fails in a debug build
#ifdef _DEBUG
#define adgASSERT(x) if (!(x)) adgASSERTFAIL(__FILE__, __LINE__, #x)
#else
#define adgASSERT(x)
#endif

// Assert in debug builds, but don't remove the code in retail builds
#ifdef _DEBUG
#define adgVERIFY(x) adgASSERT(x)
#else
#define adgVERIFY(x) (x)
#endif


/////////////////////////// adgHANDLE_DLGMSG Macro ////////////////////////////


// The normal HANDLE_MSG macro in WINDOWSX.H does not work properly for dialog
// boxes because DlgProc's return a BOOL instead of an LRESULT (like
// WndProcs). This adgHANDLE_DLGMSG macro corrects the problem:
#define adgHANDLE_DLGMSG(hwnd, message, fn)                          \
   case (message): return (SetDlgMsgResult(hwnd, uMsg,               \
      HANDLE_##message((hwnd), (wParam), (lParam), (fn))))


////////////////////////// Window Extra Byte Macros ///////////////////////////


// Macros to compute the size and offset of structure members
//#define adgMEMBEROFFSET(structure, member) (int) (&(((structure *)0)->member))
#define adgMEMBEROFFSET(structure, member) (INT_PTR) (&(((structure *)0)->member))

// Macros to compute offsets and get/set window values based on the layout of
// a structure.
#define adgSETWINDOWWORD(hwnd, structure, member, value) \
   SetWindowWord(hwnd, adgMEMBEROFFSET(structure, member), (WORD) (value))
#define adgSETWINDOWLONG(hwnd, structure, member, value) \
   SetWindowLong(hwnd, adgMEMBEROFFSET(structure, member), (LONG) (value))
#define adgGETWINDOWWORD(hwnd, structure, member) \
   GetWindowWord(hwnd, adgMEMBEROFFSET(structure, member))
#define adgGETWINDOWLONG(hwnd, structure, member) \
   GetWindowLong(hwnd, adgMEMBEROFFSET(structure, member))


/////////////////////////// Quick MessageBox Macro ////////////////////////////


#define adgMB(s) {                                                   \
      TCHAR szTMP[128];                                              \
      GetModuleFileName(NULL, szTMP, adgARRAY_SIZE(szTMP));          \
      MessageBox(GetActiveWindow(), s, szTMP, MB_OK);                \
   }


///////////////////////////// Zero Variable Macro /////////////////////////////


// Zero out a structure. If fInitSize is TRUE then initialize the first int to
// the size of the structure. Many structures like WNDCLASSEX and STARTUPINFO
// require that their first member be set to the size of the structure itself.
#define adgINITSTRUCT(structure, fInitSize)                          \
   (ZeroMemory(&(structure), sizeof(structure)),                     \
   fInitSize ? (*(int*) &(structure) = sizeof(structure)) : 0)


//////////////////////// Dialog Box Icon Setting Macro ////////////////////////


// The call to SetClassLong is for Windows NT 3.51 or less.  The WM_SETICON
// messages are for Windows 95 and future versions of NT.
#define adgSETDLGICONS(hwnd, idiLarge, idiSmall)                              \
   {                                                                          \
      OSVERSIONINFO VerInfo;                                                  \
      adgINITSTRUCT(VerInfo, TRUE);                                           \
      GetVersionEx(&VerInfo);                                                 \
      if ((VerInfo.dwPlatformId == VER_PLATFORM_WIN32_NT) &&                  \
          (VerInfo.dwMajorVersion <= 3 && VerInfo.dwMinorVersion <= 51)) {    \
         SetClassLong(hwnd, GCL_HICON, (LONG)                                 \
            LoadIcon(GetWindowInstance(hwnd), MAKEINTRESOURCE(idiLarge)));    \
      } else {                                                                \
         SendMessage(hwnd, WM_SETICON, TRUE,  (LPARAM)                        \
            LoadIcon(GetWindowInstance(hwnd), MAKEINTRESOURCE(idiLarge)));    \
         SendMessage(hwnd, WM_SETICON, FALSE, (LPARAM)                        \
            LoadIcon(GetWindowInstance(hwnd), MAKEINTRESOURCE(idiSmall)));    \
      }                                                                       \
   }
    

///////////////////////////// UNICODE Check Macro /////////////////////////////


#ifdef UNICODE

#define adgWARNIFUNICODEUNDERWIN95()                                       \
   if (GetWindowsDirectoryW(NULL, 0) <= 0)                                 \
      MessageBoxA(NULL, "This operating system doesn't support Unicode.",  \
         NULL, MB_OK)

#else

#define adgWARNIFUNICODEUNDERWIN95()

#endif



///////////////////////////////// End of File /////////////////////////////////
